@@ -1,11 +1,26 @@
+#  Copyright 2022 Upstream Data Inc
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import ipaddress
 import logging
 import json
+from typing import Union
 
 import toml
 
 
-from pyasic.miners import BaseMiner
+from pyasic.miners.base import BaseMiner
 from pyasic.API.bosminer import BOSMinerAPI
 from pyasic.API import APIError
 
@@ -14,7 +29,7 @@ from pyasic.data import MinerData
 
 from pyasic.config import MinerConfig
 
-from pyasic.settings import MINER_FACTORY_GET_VERSION_RETRIES as DATA_RETRIES
+from pyasic.settings import PyasicSettings
 
 
 class BOSMiner(BaseMiner):
@@ -27,10 +42,11 @@ class BOSMiner(BaseMiner):
         self.pwd = "admin"
         self.config = None
 
-    async def send_ssh_command(self, cmd: str) -> str or None:
+    async def send_ssh_command(self, cmd: str) -> Union[str, None]:
         """Send a command to the miner over ssh.
 
-        :return: Result of the command or None.
+        Returns:
+            Result of the command or None.
         """
         result = None
 
@@ -56,11 +72,11 @@ class BOSMiner(BaseMiner):
     async def fault_light_on(self) -> bool:
         """Sends command to turn on fault light on the miner."""
         logging.debug(f"{self}: Sending fault_light on command.")
-        self.light = True
         _ret = await self.send_ssh_command("miner fault_light on")
         logging.debug(f"{self}: fault_light on command completed.")
         if isinstance(_ret, str):
-            return True
+            self.light = True
+            return self.light
         return False
 
     async def fault_light_off(self) -> bool:
@@ -70,10 +86,12 @@ class BOSMiner(BaseMiner):
         _ret = await self.send_ssh_command("miner fault_light off")
         logging.debug(f"{self}: fault_light off command completed.")
         if isinstance(_ret, str):
+            self.light = False
             return True
         return False
 
     async def restart_backend(self) -> bool:
+        """Restart bosminer hashing process.  Wraps [`restart_bosminer`][pyasic.miners._backends.bosminer.BOSMiner.restart_bosminer] to standardize."""
         return await self.restart_bosminer()
 
     async def restart_bosminer(self) -> bool:
@@ -94,7 +112,12 @@ class BOSMiner(BaseMiner):
             return True
         return False
 
-    async def get_config(self) -> None:
+    async def get_config(self) -> MinerConfig:
+        """Gets the config for the miner and sets it as `self.config`.
+
+        Returns:
+            The config from `self.config`.
+        """
         logging.debug(f"{self}: Getting config.")
         async with (await self._get_ssh_connection()) as conn:
             logging.debug(f"{self}: Opening SFTP connection.")
@@ -110,7 +133,8 @@ class BOSMiner(BaseMiner):
     async def get_hostname(self) -> str:
         """Get miner hostname.
 
-        :return: The hostname of the miner as a string or "?"
+        Returns:
+            The hostname of the miner as a string or "?"
         """
         if self.hostname:
             return self.hostname
@@ -129,10 +153,11 @@ class BOSMiner(BaseMiner):
             logging.warning(f"Failed to get hostname for miner: {self}")
             return "?"
 
-    async def get_model(self) -> str or None:
+    async def get_model(self) -> Union[str, None]:
         """Get miner model.
 
-        :return: Miner model or None.
+        Returns:
+            Miner model or None.
         """
         # check if model is cached
         if self.model:
@@ -166,10 +191,11 @@ class BOSMiner(BaseMiner):
         logging.warning(f"Failed to get model for miner: {self}")
         return None
 
-    async def get_version(self):
+    async def get_version(self) -> Union[str, None]:
         """Get miner firmware version.
 
-        :return: Miner firmware version or None.
+        Returns:
+            Miner firmware version or None.
         """
         # check if version is cached
         if self.version:
@@ -189,82 +215,71 @@ class BOSMiner(BaseMiner):
         logging.warning(f"Failed to get model for miner: {self}")
         return None
 
-    async def send_config(self, yaml_config, ip_user: bool = False) -> None:
+    async def send_config(self, config: MinerConfig, user_suffix: str = None) -> None:
         """Configures miner with yaml config."""
         logging.debug(f"{self}: Sending config.")
-        if ip_user:
-            suffix = str(self.ip).split(".")[-1]
-            toml_conf = (
-                MinerConfig()
-                .from_yaml(yaml_config)
-                .as_bos(model=self.model.replace(" (BOS)", ""), user_suffix=suffix)
-            )
-        else:
-            toml_conf = (
-                MinerConfig()
-                .from_yaml(yaml_config)
-                .as_bos(model=self.model.replace(" (BOS)", ""))
-            )
+        toml_conf = config.as_bos(
+            model=self.model.replace(" (BOS)", ""), user_suffix=user_suffix
+        )
         async with (await self._get_ssh_connection()) as conn:
+            await conn.run("/etc/init.d/bosminer stop")
             logging.debug(f"{self}: Opening SFTP connection.")
             async with conn.start_sftp_client() as sftp:
                 logging.debug(f"{self}: Opening config file.")
                 async with sftp.open("/etc/bosminer.toml", "w+") as file:
                     await file.write(toml_conf)
             logging.debug(f"{self}: Restarting BOSMiner")
-            await conn.run("/etc/init.d/bosminer restart")
+            await conn.run("/etc/init.d/bosminer start")
 
-    async def get_board_info(self) -> dict:
-        """Gets data on each board and chain in the miner."""
-        logging.debug(f"{self}: Getting board info.")
-        devdetails = await self.api.devdetails()
-        if not devdetails.get("DEVDETAILS"):
-            print("devdetails error", devdetails)
-            return {0: [], 1: [], 2: []}
-        devs = devdetails["DEVDETAILS"]
-        boards = {}
-        offset = devs[0]["ID"]
-        for board in devs:
-            boards[board["ID"] - offset] = []
-            if not board["Chips"] == self.nominal_chips:
-                nominal = False
-            else:
-                nominal = True
-            boards[board["ID"] - offset].append(
-                {
-                    "chain": board["ID"] - offset,
-                    "chip_count": board["Chips"],
-                    "chip_status": "o" * board["Chips"],
-                    "nominal": nominal,
-                }
-            )
-        logging.debug(f"Found board data for {self}: {boards}")
-        return boards
+    async def check_light(self) -> bool:
+        if not self.light:
+            self.light = False
+        return self.light
 
-    async def get_bad_boards(self) -> dict:
-        """Checks for and provides list of non working boards."""
-        boards = await self.get_board_info()
-        bad_boards = {}
-        for board in boards.keys():
-            for chain in boards[board]:
-                if not chain["chip_count"] == 63:
-                    if board not in bad_boards.keys():
-                        bad_boards[board] = []
-                    bad_boards[board].append(chain)
-        return bad_boards
+    async def get_errors(self) -> list:
+        tunerstatus = None
+        errors = []
 
-    async def check_good_boards(self) -> str:
-        """Checks for and provides list for working boards."""
-        devs = await self.api.devdetails()
-        bad = 0
-        chains = devs["DEVDETAILS"]
-        for chain in chains:
-            if chain["Chips"] == 0:
-                bad += 1
-        if not bad > 0:
-            return str(self.ip)
+        try:
+            tunerstatus = await self.api.tunerstatus()
+        except Exception as e:
+            logging.warning(e)
+
+        if tunerstatus:
+            tuner = tunerstatus[0].get("TUNERSTATUS")
+            if tuner:
+                if len(tuner) > 0:
+                    chain_status = tuner[0].get("TunerChainStatus")
+                    if chain_status and len(chain_status) > 0:
+                        board_map = {
+                            0: "Left board",
+                            1: "Center board",
+                            2: "Right board",
+                        }
+                        offset = (
+                            6
+                            if chain_status[0]["HashchainIndex"] in [6, 7, 8]
+                            else chain_status[0]["HashchainIndex"]
+                        )
+                        for board in chain_status:
+                            _id = board["HashchainIndex"] - offset
+                            if board["Status"] not in [
+                                "Stable",
+                                "Testing performance profile",
+                            ]:
+                                _error = board["Status"]
+                                _error = _error[0].lower() + _error[1:]
+                                errors.append(
+                                    BraiinsOSError(f"{board_map[_id]} {_error}")
+                                )
+        return errors
 
     async def get_data(self) -> MinerData:
+        """Get data from the miner.
+
+        Returns:
+            A [`MinerData`][pyasic.data.MinerData] instance containing the miners data.
+        """
         data = MinerData(ip=str(self.ip), ideal_chips=self.nominal_chips * 3)
 
         board_offset = -1
@@ -283,8 +298,10 @@ class BOSMiner(BaseMiner):
         if mac:
             data.mac = mac
 
+        data.fault_light = await self.check_light()
+
         miner_data = None
-        for i in range(DATA_RETRIES):
+        for i in range(PyasicSettings().miner_get_data_retries):
             try:
                 miner_data = await self.api.multicommand(
                     "summary",
