@@ -1,6 +1,21 @@
-from typing import TypeVar, Tuple, List
+#  Copyright 2022 Upstream Data Inc
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
+from typing import Tuple, List, Union
 from collections.abc import AsyncIterable
-from pyasic.miners import BaseMiner
+from pyasic.miners.base import AnyMiner
+import httpx
 
 from pyasic.miners.antminer import *
 from pyasic.miners.avalonminer import *
@@ -10,9 +25,9 @@ from pyasic.miners._backends.cgminer import CGMiner  # noqa - Ignore _module imp
 from pyasic.miners._backends.bmminer import BMMiner  # noqa - Ignore _module import
 from pyasic.miners._backends.bosminer import BOSMiner  # noqa - Ignore _module import
 from pyasic.miners._backends.btminer import BTMiner  # noqa - Ignore _module import
-from pyasic.miners._backends.bosminer_old import (
+from pyasic.miners._backends.bosminer_old import (  # noqa - Ignore _module import
     BOSMinerOld,
-)  # noqa - Ignore _module import
+)
 
 from pyasic.miners.unknown import UnknownMiner
 
@@ -23,106 +38,113 @@ import ipaddress
 import json
 import logging
 
-from pyasic.settings import (
-    MINER_FACTORY_GET_VERSION_RETRIES as GET_VERSION_RETRIES,
-    NETWORK_PING_TIMEOUT as PING_TIMEOUT,
-)
+from pyasic.settings import PyasicSettings
 
 import asyncssh
 
-AnyMiner = TypeVar("AnyMiner", bound=BaseMiner)
-
 MINER_CLASSES = {
-    "Antminer S9": {
+    "ANTMINER S9": {
         "Default": BOSMinerS9,
         "BOSMiner": BOSMinerOld,
         "BOSMiner+": BOSMinerS9,
         "BMMiner": BMMinerS9,
         "CGMiner": CGMinerS9,
     },
-    "Antminer S9i": {
+    "ANTMINER S9I": {
         "Default": BMMinerS9i,
         "BMMiner": BMMinerS9i,
     },
-    "Antminer S17": {
+    "ANTMINER T9": {
+        "Default": BMMinerT9,
+        "BMMiner": BMMinerT9,
+        "Hiveon": HiveonT9,
+    },
+    "ANTMINER S17": {
         "Default": BMMinerS17,
         "BOSMiner+": BOSMinerS17,
         "BMMiner": BMMinerS17,
         "CGMiner": CGMinerS17,
     },
-    "Antminer S17+": {
+    "ANTMINER S17+": {
         "Default": BMMinerS17Plus,
         "BOSMiner+": BOSMinerS17Plus,
         "BMMiner": BMMinerS17Plus,
         "CGMiner": CGMinerS17Plus,
     },
-    "Antminer S17 Pro": {
+    "ANTMINER S17 PRO": {
         "Default": BMMinerS17Pro,
         "BOSMiner+": BOSMinerS17Pro,
         "BMMiner": BMMinerS17Pro,
         "CGMiner": CGMinerS17Pro,
     },
-    "Antminer S17e": {
+    "ANTMINER S17E": {
         "Default": BMMinerS17e,
         "BOSMiner+": BOSMinerS17e,
         "BMMiner": BMMinerS17e,
         "CGMiner": CGMinerS17e,
     },
-    "Antminer T17": {
+    "ANTMINER T17": {
         "Default": BMMinerT17,
         "BOSMiner+": BOSMinerT17,
         "BMMiner": BMMinerT17,
         "CGMiner": CGMinerT17,
     },
-    "Antminer T17+": {
+    "ANTMINER T17+": {
         "Default": BMMinerT17Plus,
         "BOSMiner+": BOSMinerT17Plus,
         "BMMiner": BMMinerT17Plus,
         "CGMiner": CGMinerT17Plus,
     },
-    "Antminer T17e": {
+    "ANTMINER T17E": {
         "Default": BMMinerT17e,
         "BOSMiner+": BOSMinerT17e,
         "BMMiner": BMMinerT17e,
         "CGMiner": CGMinerT17e,
     },
-    "Antminer S19": {
+    "ANTMINER S19": {
         "Default": BMMinerS19,
         "BOSMiner+": BOSMinerS19,
         "BMMiner": BMMinerS19,
         "CGMiner": CGMinerS19,
     },
-    "Antminer S19 Pro": {
+    "ANTMINER S19 PRO": {
         "Default": BMMinerS19Pro,
         "BOSMiner+": BOSMinerS19Pro,
         "BMMiner": BMMinerS19Pro,
         "CGMiner": CGMinerS19Pro,
     },
-    "Antminer S19j": {
+    "ANTMINER S19J": {
         "Default": BMMinerS19j,
         "BOSMiner+": BOSMinerS19j,
         "BMMiner": BMMinerS19j,
         "CGMiner": CGMinerS19j,
     },
-    "Antminer S19j Pro": {
+    "ANTMINER S19J PRO": {
         "Default": BMMinerS19jPro,
         "BOSMiner+": BOSMinerS19jPro,
         "BMMiner": BMMinerS19jPro,
         "CGMiner": CGMinerS19jPro,
     },
-    "Antminer S19a": {
+    "ANTMINER S19A": {
         "Default": BMMinerS19a,
         "BMMiner": BMMinerS19a,
     },
-    "Antminer T19": {
+    "ANTMINER T19": {
         "Default": BMMinerT19,
         "BOSMiner+": BOSMinerT19,
         "BMMiner": BMMinerT19,
         "CGMiner": CGMinerT19,
     },
+    "M20": {
+        "Default": BTMinerM20,
+        "BTMiner": BTMinerM20,
+        "10": BTMinerM20V10,
+    },
     "M20S": {
         "Default": BTMinerM20S,
         "BTMiner": BTMinerM20S,
+        "10": BTMinerM20SV10,
+        "20": BTMinerM20SV20,
     },
     "M20S+": {
         "Default": BTMinerM20SPlus,
@@ -176,46 +198,52 @@ MINER_CLASSES = {
         "Default": BTMinerM32S,
         "BTMiner": BTMinerM32S,
     },
-    "AvalonMiner 721": {
+    "M32": {
+        "Default": BTMinerM32,
+        "BTMiner": BTMinerM32,
+        "20": BTMinerM32V20,
+    },
+    "AVALONMINER 721": {
         "Default": CGMinerAvalon721,
         "CGMiner": CGMinerAvalon721,
     },
-    "AvalonMiner 741": {
+    "AVALONMINER 741": {
         "Default": CGMinerAvalon741,
         "CGMiner": CGMinerAvalon741,
     },
-    "AvalonMiner 761": {
+    "AVALONMINER 761": {
         "Default": CGMinerAvalon761,
         "CGMiner": CGMinerAvalon761,
     },
-    "AvalonMiner 821": {
+    "AVALONMINER 821": {
         "Default": CGMinerAvalon821,
         "CGMiner": CGMinerAvalon821,
     },
-    "AvalonMiner 841": {
+    "AVALONMINER 841": {
         "Default": CGMinerAvalon841,
         "CGMiner": CGMinerAvalon841,
     },
-    "AvalonMiner 851": {
+    "AVALONMINER 851": {
         "Default": CGMinerAvalon851,
         "CGMiner": CGMinerAvalon851,
     },
-    "AvalonMiner 921": {
+    "AVALONMINER 921": {
         "Default": CGMinerAvalon921,
         "CGMiner": CGMinerAvalon921,
     },
-    "AvalonMiner 1026": {
+    "AVALONMINER 1026": {
         "Default": CGMinerAvalon1026,
         "CGMiner": CGMinerAvalon1026,
     },
-    "AvalonMiner 1047": {
+    "AVALONMINER 1047": {
         "Default": CGMinerAvalon1047,
         "CGMiner": CGMinerAvalon1047,
     },
-    "AvalonMiner 1066": {
+    "AVALONMINER 1066": {
         "Default": CGMinerAvalon1066,
         "CGMiner": CGMinerAvalon1066,
     },
+    "Unknown": {"Default": UnknownMiner},
 }
 
 
@@ -229,11 +257,13 @@ class Singleton(type):
 
 
 class MinerFactory(metaclass=Singleton):
+    """A factory to handle identification and selection of the proper class of miner"""
+
     def __init__(self) -> None:
         self.miners = {}
 
     async def get_miner_generator(
-        self, ips: List[ipaddress.ip_address or str]
+        self, ips: List[Union[ipaddress.ip_address, str]]
     ) -> AsyncIterable[AnyMiner]:
         """
         Get Miner objects from ip addresses using an async generator.
@@ -242,6 +272,9 @@ class MinerFactory(metaclass=Singleton):
 
         Parameters:
             ips: a list of ip addresses to get miners for.
+
+        Returns:
+            An async iterable containing miners.
         """
         # get the event loop
         loop = asyncio.get_event_loop()
@@ -256,27 +289,30 @@ class MinerFactory(metaclass=Singleton):
         for miner in scanned:
             yield await miner
 
-    async def get_miner(self, ip: ipaddress.ip_address or str) -> AnyMiner:
-        """Decide a miner type using the IP address of the miner."""
+    async def get_miner(self, ip: Union[ipaddress.ip_address, str]) -> AnyMiner:
+        """Decide a miner type using the IP address of the miner.
+
+        Parameters:
+            ip: An `ipaddress.ip_address` or string of the IP to find the miner.
+
+        Returns:
+            A miner class.
+        """
         if isinstance(ip, str):
             ip = ipaddress.ip_address(ip)
         # check if the miner already exists in cache
         if ip in self.miners:
             return self.miners[ip]
         # if everything fails, the miner is already set to unknown
-        miner = UnknownMiner(str(ip))
-        api = None
-        model = None
-        ver = None
+        model, api, ver = None, None, None
 
         # try to get the API multiple times based on retries
-        for i in range(GET_VERSION_RETRIES):
+        for i in range(PyasicSettings().miner_factory_get_version_retries):
             try:
                 # get the API type, should be BOSMiner, CGMiner, BMMiner, BTMiner, or None
                 new_model, new_api, new_ver = await asyncio.wait_for(
-                    self._get_miner_type(ip), timeout=PING_TIMEOUT
+                    self._get_miner_type(ip), timeout=10
                 )
-
                 # keep track of the API and model we found first
                 if new_api and not api:
                     api = new_api
@@ -284,13 +320,29 @@ class MinerFactory(metaclass=Singleton):
                     model = new_model
                 if new_ver and not ver:
                     ver = new_ver
-
                 # if we find the API and model, don't need to loop anymore
                 if api and model:
                     break
             except asyncio.TimeoutError:
-                pass
+                logging.warning(f"{ip}: Get Miner Timed Out")
 
+        miner = self._select_miner_from_classes(ip, model, api, ver)
+
+        # save the miner to the cache at its IP if its not unknown
+        if not isinstance(miner, UnknownMiner):
+            self.miners[ip] = miner
+
+        # return the miner
+        return miner
+
+    @staticmethod
+    def _select_miner_from_classes(
+        ip: ipaddress.ip_address,
+        model: Union[str, None],
+        api: Union[str, None],
+        ver: Union[str, None],
+    ) -> AnyMiner:
+        miner = UnknownMiner(str(ip))
         # make sure we have model information
         if model:
             if not api:
@@ -298,12 +350,10 @@ class MinerFactory(metaclass=Singleton):
 
             if model not in MINER_CLASSES.keys():
                 if "avalon" in model:
-                    print(model)
                     if model == "avalon10":
                         miner = CGMinerAvalon1066(str(ip))
                     else:
                         miner = CGMinerAvalon821(str(ip))
-                miner = UnknownMiner(str(ip))
                 return miner
             if api not in MINER_CLASSES[model].keys():
                 api = "Default"
@@ -328,11 +378,6 @@ class MinerFactory(metaclass=Singleton):
                 elif "BMMiner" in api:
                     miner = BMMiner(str(ip))
 
-        # save the miner to the cache at its IP if its not unknown
-        if not isinstance(miner, UnknownMiner):
-            self.miners[ip] = miner
-
-        # return the miner
         return miner
 
     def clear_cached_miners(self) -> None:
@@ -341,173 +386,198 @@ class MinerFactory(metaclass=Singleton):
         self.miners = {}
 
     async def _get_miner_type(
-        self, ip: ipaddress.ip_address or str
-    ) -> Tuple[str or None, str or None, str or None]:
-        model = None
-        api = None
-        ver = None
-
-        devdetails = None
-        version = None
+        self, ip: Union[ipaddress.ip_address, str]
+    ) -> Tuple[Union[str, None], Union[str, None], Union[str, None]]:
+        model, api, ver = None, None, None
 
         try:
-            # get device details and version data
-            data = await self._send_api_command(str(ip), "devdetails+version")
-
-            # validate success
-            validation = await self._validate_command(data)
-            if not validation[0]:
-                raise APIError(validation[1])
-
-            # copy each part of the main command to devdetails and version
-            devdetails = data["devdetails"][0]
-            version = data["version"][0]
-
-        except APIError:
-            # if getting data fails we need to check again
-            data = None
-
-        # if data is None then get it a slightly different way
-        if not data:
+            devdetails, version = await self.__get_devdetails_and_version(ip)
+        except APIError as e:
+            # catch APIError and let the factory know we cant get data
+            logging.warning(f"{ip}: API Command Error: {e}")
+            return None, None, None
+        except OSError or ConnectionRefusedError:
+            # miner refused connection on API port, we wont be able to get data this way
+            # try ssh
             try:
-                # try devdetails and version separately (X19s mainly require this)
-                # get devdetails and validate
-                devdetails = await self._send_api_command(str(ip), "devdetails")
-                validation = await self._validate_command(devdetails)
-                if not validation[0]:
-                    # if devdetails fails try version instead
-                    devdetails = None
-
-                    # get version and validate
-                    version = await self._send_api_command(str(ip), "version")
-                    validation = await self._validate_command(version)
-                    if not validation[0]:
-                        # finally try get_version (Whatsminers) and validate
-                        version = await self._send_api_command(str(ip), "get_version")
-                        validation = await self._validate_command(version)
-
-                        # if this fails we raise an error to be caught below
-                        if not validation[0]:
-                            raise APIError(validation[1])
-            except APIError as e:
-                # catch APIError and let the factory know we cant get data
-                logging.warning(f"{ip}: API Command Error: {e}")
-                return None, None, None
+                _model = await self.__get_model_from_ssh(ip)
+                if _model:
+                    model = _model
+                    api = "BOSMiner+"
+            except asyncssh.misc.PermissionDenied:
+                try:
+                    data = await self.__get_system_info_from_web(ip)
+                    if "minertype" in data.keys():
+                        model = data["minertype"].upper()
+                    if "bmminer" in "\t".join(data.keys()):
+                        api = "BMMiner"
+                except Exception as e:
+                    logging.debug(f"Unable to get miner - {e}")
+            return model, api, ver
 
         # if we have devdetails, we can get model data from there
         if devdetails:
-            if "DEVDETAILS" in devdetails.keys() and not devdetails["DEVDETAILS"] == []:
-                # check for model, for most miners
-                if not devdetails["DEVDETAILS"][0]["Model"] == "":
-                    # model of most miners
-                    model = devdetails["DEVDETAILS"][0]["Model"]
-
-                # if model fails, try driver
-                else:
-                    # some avalonminers have model in driver
-                    model = devdetails["DEVDETAILS"][0]["Driver"]
-            else:
+            for _devdetails_key in ["Model", "Driver"]:
+                try:
+                    model = devdetails["DEVDETAILS"][0][_devdetails_key].upper()
+                    if not model == "BITMICRO":
+                        break
+                except KeyError:
+                    continue
+            if not model:
+                # braiins OS bug check just in case
                 if "s9" in devdetails["STATUS"][0]["Description"]:
-                    model = "Antminer S9"
+                    model = "ANTMINER S9"
+                if "s17" in version["STATUS"][0]["Description"]:
+                    model = "ANTMINER S17"
 
         # if we have version we can get API type from here
         if version:
-            if "VERSION" in version.keys():
-                # check if there are any BMMiner strings in any of the dict keys
-                if any("BMMiner" in string for string in version["VERSION"][0].keys()):
-                    api = "BMMiner"
-
-                # check if there are any CGMiner strings in any of the dict keys
-                elif any(
-                    "CGMiner" in string for string in version["VERSION"][0].keys()
-                ):
-                    api = "CGMiner"
-
-                elif any(
-                    "BTMiner" in string for string in version["VERSION"][0].keys()
-                ):
-                    api = "BTMiner"
+            if "VERSION" in version:
+                api_types = ["BMMiner", "CGMiner", "BTMiner"]
+                # check basic API types, BOSMiner needs a special check
+                for api_type in api_types:
+                    if any(api_type in string for string in version["VERSION"][0]):
+                        api = api_type
 
                 # check if there are any BOSMiner strings in any of the dict keys
-                elif any(
-                    "BOSminer" in string for string in version["VERSION"][0].keys()
-                ):
+                if any("BOSminer" in string for string in version["VERSION"][0]):
                     api = "BOSMiner"
                     if version["VERSION"][0].get("BOSminer"):
                         if "plus" in version["VERSION"][0]["BOSminer"]:
                             api = "BOSMiner+"
-
-                    if "BOSminer+" in version["VERSION"][0].keys():
+                    if "BOSminer+" in version["VERSION"][0]:
                         api = "BOSMiner+"
 
-            # if all that fails, check the Description to see if it is a whatsminer
+                # check for avalonminers
+                for _version_key in ["PROD", "MODEL"]:
+                    try:
+                        _data = version["VERSION"][0][_version_key].split("-")
+                    except KeyError:
+                        continue
+
+                    model = _data[0].upper()
+                    if _version_key == "MODEL":
+                        model = f"AVALONMINER {_data[0]}"
+                    if len(_data) > 1:
+                        ver = _data[1]
+
             if version.get("Description") and (
                 "whatsminer" in version.get("Description")
             ):
                 api = "BTMiner"
 
-            # check for avalonminers
-            if version["VERSION"][0].get("PROD"):
-                _data = version["VERSION"][0]["PROD"].split("-")
-                model = _data[0]
-                if len(data) > 1:
-                    ver = _data[1]
-            elif version["VERSION"][0].get("MODEL"):
-                _data = version["VERSION"][0]["MODEL"].split("-")
-                model = f"AvalonMiner {_data[0]}"
-                if len(data) > 1:
-                    ver = _data[1]
-
         # if we have no model from devdetails but have version, try to get it from there
         if version and not model:
-            # make sure version isn't blank
-            if (
-                "VERSION" in version.keys()
-                and version.get("VERSION")
-                and not version.get("VERSION") == []
-            ):
-                # try to get "Type" which is model
-                if version["VERSION"][0].get("Type"):
-                    model = version["VERSION"][0]["Type"]
+            try:
+                model = version["VERSION"][0]["Type"].upper()
+            except KeyError:
+                pass
 
-                # braiins OS bug check just in case
-                elif "am2-s17" in version["STATUS"][0]["Description"]:
-                    model = "Antminer S17"
-
-                # final try on a braiins OS bug with devdetails not returning
+        if not model:
+            stats = await self._send_api_command(str(ip), "stats")
+            if stats:
+                try:
+                    _model = stats["STATS"][0]["Type"].upper()
+                except KeyError:
+                    pass
                 else:
-                    try:
-                        async with asyncssh.connect(
-                            str(ip),
-                            known_hosts=None,
-                            username="root",
-                            password="admin",
-                            server_host_key_algs=["ssh-rsa"],
-                        ) as conn:
-                            cfg = await conn.run("bosminer config --data")
-                        if cfg:
-                            cfg = json.loads(cfg.stdout)
-                            model = cfg.get("data").get("format").get("model")
-                    except asyncssh.misc.PermissionDenied:
-                        pass
+                    for split_point in [" BB", " XILINX", " (VNISH"]:
+                        if split_point in _model:
+                            _model = _model.split(split_point)[0]
+                    if "PRO" in _model and " PRO" not in _model:
+                        _model = _model.replace("PRO", " PRO")
+                    model = _model
 
         if model:
-            # whatsminer have a V in their version string (M20SV41), remove everything after it
+            if " HIVEON" in model:
+                # do hiveon check before whatsminer as HIVEON contains a V
+                model = model.split(" HIVEON")[0]
+                api = "Hiveon"
+            # whatsminer have a V in their version string (M20SV41), everything after it is ver
             if "V" in model:
                 _ver = model.split("V")
                 if len(_ver) > 1:
                     ver = model.split("V")[1]
                     model = model.split("V")[0]
-            # don't need "Bitmain", just "Antminer XX" as model
-            if "Bitmain " in model:
-                model = model.replace("Bitmain ", "")
+            # don't need "Bitmain", just "ANTMINER XX" as model
+            if "BITMAIN " in model:
+                model = model.replace("BITMAIN ", "")
+
         return model, api, ver
 
+    async def __get_devdetails_and_version(
+        self, ip
+    ) -> Tuple[Union[dict, None], Union[dict, None]]:
+        version = None
+        try:
+            # get device details and version data
+            data = await self._send_api_command(str(ip), "devdetails+version")
+            # validate success
+            validation = await self._validate_command(data)
+            if not validation[0]:
+                raise APIError(validation[1])
+            # copy each part of the main command to devdetails and version
+            devdetails = data["devdetails"][0]
+            version = data["version"][0]
+            return devdetails, version
+        except APIError:
+            # try devdetails and version separately (X19s mainly require this)
+            # get devdetails and validate
+            devdetails = await self._send_api_command(str(ip), "devdetails")
+            validation = await self._validate_command(devdetails)
+            if not validation[0]:
+                # if devdetails fails try version instead
+                devdetails = None
+
+                # get version and validate
+                version = await self._send_api_command(str(ip), "version")
+                validation = await self._validate_command(version)
+                if not validation[0]:
+                    # finally try get_version (Whatsminers) and validate
+                    version = await self._send_api_command(str(ip), "get_version")
+                    validation = await self._validate_command(version)
+
+                    # if this fails we raise an error to be caught below
+                    if not validation[0]:
+                        raise APIError(validation[1])
+        return devdetails, version
+
     @staticmethod
-    async def _validate_command(data: dict) -> Tuple[bool, str or None]:
+    async def __get_model_from_ssh(ip: ipaddress.ip_address) -> Union[str, None]:
+        model = None
+        async with asyncssh.connect(
+            str(ip),
+            known_hosts=None,
+            username="root",
+            password="admin",
+            server_host_key_algs=["ssh-rsa"],
+        ) as conn:
+            board_name = None
+            cmd = await conn.run("cat /tmp/sysinfo/board_name")
+            if cmd:
+                board_name = cmd.stdout.strip()
+        if board_name == "am1-s9":
+            model = "ANTMINER S9"
+        if board_name == "am2-s17":
+            model = "ANTMINER S17"
+        return model
+
+    @staticmethod
+    async def __get_system_info_from_web(ip) -> dict:
+        url = f"http://{ip}/cgi-bin/get_system_info.cgi"
+        auth = httpx.DigestAuth("root", "root")
+        async with httpx.AsyncClient() as client:
+            data = await client.get(url, auth=auth)
+        if data.status_code == 200:
+            data = data.json()
+        return data
+
+    @staticmethod
+    async def _validate_command(data: dict) -> Tuple[bool, Union[str, None]]:
         """Check if the returned command output is correctly formatted."""
         # check if the data returned is correct or an error
-        if not data:
+        if not data or data == {}:
             return False, "No API data."
         # if status isn't a key, it is a multicommand
         if "STATUS" not in data.keys():
@@ -531,14 +601,17 @@ class MinerFactory(metaclass=Singleton):
         return True, None
 
     @staticmethod
-    async def _send_api_command(ip: ipaddress.ip_address or str, command: str) -> dict:
+    async def _send_api_command(
+        ip: Union[ipaddress.ip_address, str], command: str
+    ) -> dict:
         try:
             # get reader and writer streams
             reader, writer = await asyncio.open_connection(str(ip), 4028)
         except OSError as e:
+            if e.errno in [10061, 22]:
+                raise e
             logging.warning(f"{str(ip)} - Command {command}: {e}")
             return {}
-
         # create the command
         cmd = {"command": command}
 
